@@ -26,10 +26,12 @@ from async_upnp_client.const import (
     NS,
     ActionArgumentInfo,
     ActionInfo,
+    DeviceIcon,
     DeviceInfo,
     ServiceInfo,
     StateVariableInfo,
 )
+from async_upnp_client.exceptions import UpnpError, UpnpValueError
 
 _LOGGER = logging.getLogger(__name__)
 _LOGGER_TRAFFIC_UPNP = logging.getLogger("async_upnp_client.traffic.upnp")
@@ -52,7 +54,7 @@ class UpnpRequester:
         headers: Optional[Mapping[str, str]] = None,
         body: Optional[str] = None,
         body_type: str = "text",
-    ) -> Tuple[int, Mapping, Union[str, bytes, None]]:
+    ) -> Tuple[int, Mapping[str, str], Union[str, bytes, None]]:
         """
         Do a HTTP request.
 
@@ -99,24 +101,12 @@ class UpnpRequester:
         headers: Optional[Mapping[str, str]] = None,
         body: Optional[str] = None,
         body_type: str = "text",
-    ) -> Tuple[int, Mapping, Union[str, bytes, None]]:
+    ) -> Tuple[int, Mapping[str, str], Union[str, bytes, None]]:
         """Actually do a HTTP request."""
         # pylint: disable=too-many-arguments
         raise NotImplementedError()
 
 
-class UpnpError(Exception):
-    """UpnpError."""
-
-
-class UpnpValueError(UpnpError):
-    """Invalid value error."""
-
-    def __init__(self, name: str, value: Any) -> None:
-        """Initialize."""
-        super().__init__("Invalid value for %s: '%s'" % (name, value))
-
-
 class UpnpDevice:
     """UPnP Device representation."""
 
@@ -194,6 +184,11 @@ class UpnpDevice:
         """Get the device type of this device."""
         return self._device_info.device_type
 
+    @property
+    def icons(self) -> Sequence[DeviceIcon]:
+        """Get the icons for this device."""
+        return self._device_info.icons
+
     @property
     def xml(self) -> ET.Element:
         """Get the XML description for this device."""
@@ -632,10 +627,9 @@ class UpnpAction:
         query = ".//{{{0}}}{1}Response".format(service_type, self.name)
         response = xml.find(query, NS)
 
-        # Strip version number from service type to be more tolerant to incomplete namespace.
-        if response is None and ":" in service_type and self._non_strict:
-            service_type_funny = ":".join(service_type.split(":")[:-1])
-            query = ".//{{{0}}}{1}Response".format(service_type_funny, self.name)
+        # If no response was found, do a search ignoring namespaces when in non-strict mode.
+        if response is None and self._non_strict:
+            query = ".//{{*}}{0}Response".format(self.name)
             response = xml.find(query, NS)
 
         if response is None:
