@@ -1,15 +1,21 @@
 # -*- coding: utf-8 -*-
 """Utils for async_upnp_client."""
 
+import asyncio
 import re
+import socket
 from collections.abc import Mapping as abcMapping
 from collections.abc import MutableMapping
 from datetime import datetime, timedelta, timezone
-from typing import Any, Callable, Dict, Generator, Mapping, Optional
-from urllib.parse import urljoin
+from socket import AddressFamily  # pylint: disable=no-name-in-module
+from typing import Any, Callable, Dict, Generator, Mapping, Optional, Tuple
+from urllib.parse import urljoin, urlsplit
 
 from voluptuous import Invalid
 
+EXTERNAL_IP = "1.1.1.1"
+EXTERNAL_PORT = 80
+
 
 class CaseInsensitiveDict(MutableMapping):
     """Case insensitive dict."""
@@ -161,3 +167,53 @@ def parse_date_time(value: str) -> Any:
         if re.match(pattern, value):
             return parser(value)
     raise ValueError("Unknown date/time: " + value)
+
+
+def _target_url_to_addr(target_url: Optional[str]) -> Tuple[str, int]:
+    """Resolve target_url into an address usable for get_local_ip."""
+    if target_url:
+        if "//" not in target_url:
+            # Make sure urllib can work with target_url to get the host
+            target_url = "//" + target_url
+        target_url_split = urlsplit(target_url)
+        target_host = target_url_split.hostname or EXTERNAL_IP
+        target_port = target_url_split.port or EXTERNAL_PORT
+    else:
+        target_host = EXTERNAL_IP
+        target_port = EXTERNAL_PORT
+
+    return target_host, target_port
+
+
+def get_local_ip(target_url: Optional[str] = None) -> str:
+    """Try to get the local IP of this machine, used to talk to target_url."""
+    target_addr = _target_url_to_addr(target_url)
+
+    try:
+        temp_sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        temp_sock.connect(target_addr)
+        local_ip: str = temp_sock.getsockname()[0]
+        return local_ip
+    finally:
+        temp_sock.close()
+
+
+async def async_get_local_ip(
+    target_url: Optional[str] = None, loop: Optional[asyncio.AbstractEventLoop] = None
+) -> Tuple[AddressFamily, str]:
+    """Try to get the local IP of this machine, used to talk to target_url."""
+    target_addr = _target_url_to_addr(target_url)
+    loop = loop or asyncio.get_event_loop()
+
+    # Create a UDP connection to the target. This won't cause any network
+    # traffic but will assign a local IP to the socket.
+    transport, _ = await loop.create_datagram_endpoint(
+        asyncio.protocols.DatagramProtocol, remote_addr=target_addr
+    )
+
+    try:
+        sock = transport.get_extra_info("socket")
+        sockname = sock.getsockname()
+        return sock.family, sockname[0]
+    finally:
+        transport.close()
